@@ -423,6 +423,7 @@ fu_logitech_hidpp_device_fetch_firmware_info(FuLogitechHidppDevice *self, GError
 
 	/* the device is probably in bootloader mode and the last SoftDevice FW upgrade failed */
 	if (fu_device_has_private_flag(FU_DEVICE(self), FU_LOGITECH_HIDPP_DEVICE_FLAG_ADD_RADIO) &&
+	    !fu_logitech_hidpp_device_feature_get_idx(self, FU_LOGITECH_HIDPP_FEATURE_RDFU) &&
 	    !radio_ok) {
 		g_debug("no radio found, creating a fake one for recovery");
 		if (!fu_logitech_hidpp_device_create_radio_child(self, 1, 0, error)) {
@@ -798,9 +799,11 @@ fu_logitech_hidpp_device_rdfu_status_data_transfer_wait(FuLogitechHidppDevice *s
 		g_autoptr(GError) error_local = NULL;
 
 		/* wait for requested time or event */
+		/* NB: waiting longer than `rdfu_wait` breaks the specification;
+		 * unfortunately, some early firmware versions require this. */
 		if (!fu_logitech_hidpp_receive(FU_UDEV_DEVICE(self),
 					       msg_in_wait,
-					       priv->rdfu_wait * 2, /* be tolerant */
+					       priv->rdfu_wait * 3, /* be tolerant */
 					       &error_local)) {
 			if (g_error_matches(error_local, FWUPD_ERROR, FWUPD_ERROR_TIMED_OUT)) {
 				g_debug("ignored error: %s", error_local->message);
