@@ -422,7 +422,7 @@ fu_udev_backend_netlink_parse_blob(FuUdevBackend *self, GBytes *blob, GError **e
 					return TRUE;
 				if (g_strcmp0(
 					fu_udev_device_get_subsystem(FU_UDEV_DEVICE(device_tmp)),
-					"drm") != 0)
+					"drm") == 0)
 					fu_udev_backend_rescan_dpaux_devices(self);
 				g_set_object(&device_donor, FU_UDEV_DEVICE(device_tmp));
 			}
@@ -542,7 +542,7 @@ fu_udev_backend_netlink_setup(FuUdevBackend *self, GError **error)
 			    FWUPD_ERROR,
 			    FWUPD_ERROR_INTERNAL,
 			    "failed to connect to netlink: %s",
-			    g_strerror(errno));
+			    fwupd_strerror(errno));
 		return FALSE;
 	}
 	if (bind(self->netlink_fd, (void *)&nls, sizeof(nls))) {
@@ -550,7 +550,7 @@ fu_udev_backend_netlink_setup(FuUdevBackend *self, GError **error)
 			    FWUPD_ERROR,
 			    FWUPD_ERROR_INTERNAL,
 			    "bind to udev socket failed: %s",
-			    g_strerror(errno));
+			    fwupd_strerror(errno));
 		return FALSE;
 	}
 	source = g_unix_fd_source_new(self->netlink_fd, G_IO_IN);
